{
 (* whi2cllex.lex: lexical analysis for the WHILE language *)

 open Lexing whi2clpar;

 exception LexicalError of string * int * int (* (message, loc1, loc2) *)

 fun lexerError lexbuf s = 
     raise LexicalError (s, getLexemeStart lexbuf, getLexemeEnd lexbuf);

 (* Scan keywords as identifiers and use this function to distinguish them. *)
 (* If the set of keywords is large, use an auxiliary hashtable.            *)

 fun keyword s =
     case s of
         "true"         => TRUE
       | "false"        => FALSE
       | "and"          => BAND
       | "or"           => BOR
       | "not"          => BNOT
       | "skip"		=> SKIP
       | "if"           => IF
       | "then"         => THEN
       | "else"         => ELSE
       | "while"        => WHILE
       | "do"           => DO
       | _              => NAME s;
}

rule Token = parse
    [` ` `\t` `\n` `\r`]  { Token lexbuf }
  | [`a`-`z``A`-`Z`][`a`-`z``A`-`Z``0`-`9`]*
                          { keyword (getLexeme lexbuf) }
  | [`1`-`9`][`0`-`9`]* (`.`[`0`-`9`]+)?
                          { NUM (valOf (Real.fromString(getLexeme lexbuf))) }
  | `0` (`.`[`0`-`9`]+)?
                          { NUM (valOf (Real.fromString(getLexeme lexbuf))) }
  | `+`                   { ADD   } 
  | `-`                   { SUB   }
  | `*`                   { MULT  }
  | `/`                   { DIV   }
  | "=<"                  { LEQU  }
  | `=`                   { EQUA  }
  | `>`                   { GT    } 
  | ":="                  { ASGN  }
  | `;`                   { COMP  }
  | `(`                   { LPAR  }
  | `)`                   { RPAR  }
  | eof                   { EOF   }
  | _                     { lexerError lexbuf "Illegal symbol in input" }
;